##############################################################
# Innovare Studio Utilities Module
# ver. 1.1.6 / 12.Mar.2018
# Innovare Studio Inc. / D.Tsukamoto
#############################################################

import math

class vec2:
	def __init__(self, x = 0, y = 0):
		self.x = float(x)
		self.y = float(y)

	def add(self, val):
		self.x += val.x
		self.y += val.y

	def mult(self, val):
		self.x *= val.x
		self.y *= val.y

	def rotate(self, angle):
		base = vec2(self.x, self.y)
		self.x = base.x * math.cos(angle * math.pi / 180) - base.y * math.sin(angle * math.pi / 180)
		self.y = base.x * math.sin(angle * math.pi / 180) + base.y * math.cos(angle * math.pi / 180)

	def clone(self):
		return vec2(self.x, self.y)


class vecLine:
	def __init__(self, p1 = vec2(), p2 = vec2()):
		self.points = [p1, p2]

	def add(self, val):
		for point in self.points:
			point.add(val)

	def mult(self, val):
		for point in self.points:
			point.mult(val)

	def rotate(self, angle):
		for point in self.points:
			point.rotate(angle)

	def clone(self):
		return vecLine(self.points[0].clone(), self.points[1].clone())


class vecTriangle:
	def __init__(self, p1 = vec2(), p2 = vec2(), p3 = vec2()):
		self.points = [p1, p2, p3]

	def add(self, val):
		for point in self.points:
			point.add(val)

	def mult(self, val):
		for point in self.points:
			point.mult(val)

	def rotate(self, angle):
		for point in self.points:
			point.rotate(angle)

	def scale(self, val):
		x = (self.points[0].x + self.points[1].x + self.points[2].x) / 3
		y = (self.points[0].y + self.points[1].y + self.points[2].y) / 3
		for point in self.points:
			point.x -= x
			point.y -= y
			point.mult(val)
			point.x += x
			point.y += y

	def clone(self):
		return vecQuad(self.points[0].clone(), self.points[1].clone(), self.points[2].clone())


class vecQuad:
	def __init__(self, p1 = vec2(), p2 = vec2(), p3 = vec2(), p4 = vec2()):
		self.points = [p1, p2, p3, p4]

	def add(self, val):
		for point in self.points:
			point.add(val)

	def mult(self, val):
		for point in self.points:
			point.mult(val)

	def rotate(self, angle):
		for point in self.points:
			point.rotate(angle)

	def scale(self, val):
		x = (self.points[0].x + self.points[1].x + self.points[2].x + self.points[3].x) / 4
		y = (self.points[0].y + self.points[1].y + self.points[2].y + self.points[3].y) / 4
		for point in self.points:
			point.x -= x
			point.y -= y
			point.mult(val)
			point.x += x
			point.y += y

	def clone(self):
		return vecQuad(self.points[0].clone(), self.points[1].clone(), self.points[2].clone(), self.points[3].clone())


def distanceVertex(p1, p2):
	return math.pow( ( p2.x - p1.x ) * ( p2.x - p1.x ) + ( p2.y - p1.y ) * ( p2.y - p1.y ), 0.5 )


def crossVector(vl, vr):
	return vl.x * vr.y - vl.y * vr.x


def distancePointByLine(point, line):
	AB = vec2()
	AP = vec2()
	AB.x = line.p2.x - line.p1.x
	AB.y = line.p2.y - line.p1.y
	AP.x = point.x - line.p1.x
	AP.y = point.y - line.p1.y
	D = abs( crossVector( AB, AP ) )
	L = distanceVertex( line.p1, line.p2 )
	H = D / L
	return H


def slideVec2(point, angle = 0, offset = 0):
	x = point.x + math.cos(angle * math.pi / 180) * offset
	y = point.y - math.sin(angle * math.pi / 180) * offset
	return vec2(x, y)
